/*
############################################################################
# This program is used to test the basic functions of the SD file system.
# It implements simple versions of the cat, rm, ls, echo, cd, pwd, mkdir and
# rmdir commands plus the <, > and >> file redirection operators.
# The program starts up the file driver and then prompts for a command.
#
# Written by Dave Hein
# Copyright (c) 2011, 2012 Parallax, Inc.
# MIT Licensed
############################################################################
*/
#include "FFDS1.h"

#ifdef __cplusplus
extern "C" {
#endif
#include <stdio.h>
#include <propeller.h>
#include <sys/sd.h>
#include <errno.h>

extern _Driver _NullDriver;
extern _Driver _FileDriver;
_Driver *_driverlist[] =
{
    &_NullDriver,
    &_FileDriver,
    NULL
};

int32_t manual_mount()
{
    _SD_Params* mountParams = (_SD_Params*)-1;

    static _SD_Params params =
    {
        AttachmentType:
        _SDA_SingleSPI,
        pins:
        {
            SingleSPI:
            {
                MISO: 10,   // The pin attached to the SD card's MISO or DO output
                CLK:  11,   // The pin attached to the SD card's CLK or SCLK input
                MOSI: 12,   // The pin attached to the SD card's MOSI or DI input
                CS:   13    // The pin attached to the SD card's CS input
            }
        }
    };
    mountParams = &params;

    if (mountParams == (_SD_Params*)-1)
    {
        return 1;
    }

    uint32_t mountErr = dfs_mount(mountParams);
    if (mountErr)
    {
        return mountErr;
    }
    return 0;
}
#ifdef __cplusplus
}
#endif

FFDS1 debug;

const int32_t maxdur = 2*CLKFREQ; //in seconds, keep below 10 seconds to avoid clock wrap
volatile uint8_t Bigbuf[8192];

int32_t addspeed(int32_t S, int32_t Dur, int32_t N)
{
    debug.Str(S);
    debug.Tx(' ');
    debug.Dec((N + 512) / 1024);
    debug.Str(" kB in ");
    debug.Dec(Dur / (CLKFREQ/1000));
    debug.Str(" ms at ");
    debug.Dec(N / (Dur / (CLKFREQ/1024)));
    debug.Str(" kB/s\r\n");
}


int32_t pwrite_test()
{
    debug.Str("How fast can we write using ");
    debug.Str(__PRETTY_FUNCTION__);
    debug.Str("\r\n");
    FILE* fp = fopen("speed.tst", "w");

    if(fp == NULL)
    {
        debug.Str("Warning: null fp in function.\r\n");
    }
    int32_t n = 0;
    int32_t i = 2;
    int32_t duration = 0;
    while(duration < maxdur)
    {
        i += i;
        n += i;
        duration -= CNT;
        for(int x = 0; x < i; x++)
        {
            debug.Dec(x);
            debug.Tx('\r');
            debug.Tx('\n');
            fwrite(&Bigbuf, 1, 8192, fp);
        }
        duration += CNT;
    }
    fclose(fp);
    addspeed("PropGCC fwrite", duration, n*8192);

}

/* The program starts the file system. */
int main()
{

    debug.Start(31,30,230400);
    debug.Str("\r\n\r\n-----------\r\nCompile at "__TIME__", "__DATE__".\r\n");

    manual_mount();

    //Load Bigbuf with some fake data.
    memset( (void *)&Bigbuf, 0xFA, 512);

    char buf[80];
    char abe[] = "Everthing you read on the internet is true. - Abraham Lincoln\n";
    int  size = strlen(abe)+1;

    FILE* fp = fopen("abe4.txt", "w");
    if(fp)
    {
        int32_t count = fwrite(abe, 1, size, fp);
        fclose(fp);
    }
    else
    {
        debug.Str("Couldn't write to file.");
    }

    fp = fopen("abe4.txt", "r");
    if(fp)
    {
        fread(buf, 1, 80, fp);
        debug.Str(buf);
        debug.Str("\r\n");
        fclose(fp);
    }
    else
    {
        debug.Str("Couldn't read the file.");
    }

    debug.Str("Ready to start pwrite test from main.\r\n");
    pwrite_test();

    for(;;)
    {
        debug.Tx(debug.Rx());
    }

    return 0;
}

/*
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/
